#!/usr/bin/tcl

proc SplitIntoWords {block} {

    # We need to split the block up into words but cannot use
    # list operations as they throw away some significant
    # quoting, and [split] ignores braces as it should.
    # Therefore what we do is gradually build up a string out of
    # whitespace seperated strings. We cannot use [split] to
    # split the block into whitespace seperated strings as it
    # throws away the whitespace which maybe important so we
    # have to do it all by hand.

    set words {}
    set word ""

    while {[string length $block]} {
        # Look for the next group of whitespace characters.
        if {[regexp -indices "\[ \t\n\]+" $block all]} {
	    # Remove the text leading up to and including the white space
	    # from the block.
	    set text [string range $block 0 [lindex $all 1]]
	    set block [string range $block [expr {[lindex $all 1] + 1}] end]
	} else {
	    # Take everything up to the end of the block.
	    set text $block
	    set block {}
	}

	# Add the text to the end of the word we are building up.
        append word $text

        if { [catch {llength $word} length] == 0 && $length == 1} {
	    # The word is a valid list so add it to the list.
	    lappend words [string trim $word]
	    set word {}
        }
    }

    # If the last word has not been added to the list then there
    # is a problem.
    if { [string length $word] } {
	error "incomplete word \"$word\""
    }

    return $words
 }

global column
global row
global max_columns
global max_rows


global results
global parsedresultsfilename

global discharge_file_name 
global charge_file_name
global min_voltage_file_name
global peak_current_file_name
global power_on_energy_file_name
global power_off_energy_file_name

set linecount 0

set f [open $results r]
set resfile [open $parsedresultsfilename w]
set discharge_file [open $discharge_file_name a]
set charge_file [open $charge_file_name a]
set min_voltage_file [open $min_voltage_file_name a]
set peak_current_file [open $peak_current_file_name a]
set power_on_energy_file [open $power_on_energy_file_name a]
set power_off_energy_file [open $power_off_energy_file_name a]

foreach line [split [read $f] \n] {
	set wordcount 0
	if {$linecount == 6} {
	
		set words [SplitIntoWords $line]
		
		if {[string compare [lindex $words 0] "failed"] == 0} {
			puts $resfile "discharge time is: failed"
			puts -nonewline $discharge_file "0 "
		}	else {
			puts $resfile "discharge time is: [expr ([lindex $words 0]-2e-6)/1e-6] us"
			puts -nonewline $discharge_file "[expr ([lindex $words 0]-2e-6)/1e-6] "
		}
		
		if {[string compare [lindex $words 1] "failed"] == 0} {
			puts $resfile "charge time is: failed"
			puts -nonewline $charge_file "0 "
		}	else {
			puts $resfile "charge time is: [expr ([lindex $words 1]-300e-6)/1e-9] ns"
			puts -nonewline $charge_file "[expr ([lindex $words 1]-300e-6)/1e-9] "
		}
		
		if {[string compare [lindex $words 2] "failed"] == 0} {
			puts $resfile "minimum voltage is: failed"
			puts -nonewline $min_voltage_file "0 "
		}	else {
			puts $resfile "minimum voltage is: [expr ([lindex $words 2])/1e-6] uV"
			puts -nonewline $min_voltage_file "[expr ([lindex $words 2])/1e-6] "
		}
		
		if {[string compare [lindex $words 3] "failed"] == 0} {
			puts $resfile "peak current is: failed"
			puts -nonewline $peak_current_file "0 "
		}	else {
			puts $resfile "peak current is: [expr ([lindex $words 3])/1e-3] mA"
			puts -nonewline $peak_current_file "[expr ([lindex $words 3])/1e-3] "
		}							
	}
	
	if {$linecount == 7} {
		
		set words [SplitIntoWords $line]
		if {[string compare [lindex $words 3] "failed"] == 0} {
			puts $resfile "power on energy benefit is: failed"
			puts -nonewline $power_on_energy_file "0 "
		}	else {
			puts $resfile "power on energy benefit is: [expr ([lindex $words 3])/1e-15] fJ"
			puts -nonewline $power_on_energy_file "[expr ([lindex $words 3])/1e-15] "
		}				
	}
	
	if {$linecount == 8} {
		set words [SplitIntoWords $line]
		
		if {[string compare [lindex $words 2] "failed"] == 0} {
			puts $resfile "power on energy penalty is: failed"
		}	else {
			puts $resfile "power on energy penalty is: [expr ([lindex $words 2])/1e-12] pJ"
		}	
	}
	if {$linecount == 9} {
		set words [SplitIntoWords $line]		
		
		if {[string compare [lindex $words 1] "failed"] == 0} {
			puts $resfile "power off energy is: failed"
			puts -nonewline $power_off_energy_file "0 "
		}	else {
			puts $resfile "power off energy is: [expr ([lindex $words 1])/1e-12] pJ"
			puts -nonewline $power_off_energy_file "[expr ([lindex $words 1])/1e-12] "
		}	
	}
		
	incr linecount
}

if {$row == $max_rows} {
	puts -nonewline $discharge_file "\n"
	puts -nonewline $charge_file "\n"
	puts -nonewline $min_voltage_file "\n"
	puts -nonewline $peak_current_file "\n"
	puts -nonewline $power_on_energy_file "\n"
	puts -nonewline $power_off_energy_file "\n"
}
close $f
close $resfile
close $discharge_file
close $charge_file
close $min_voltage_file
close $peak_current_file
close $power_on_energy_file
close $power_off_energy_file

