#!/usr/bin/tcl

proc SplitIntoWords {block} {

    # We need to split the block up into words but cannot use
    # list operations as they throw away some significant
    # quoting, and [split] ignores braces as it should.
    # Therefore what we do is gradually build up a string out of
    # whitespace seperated strings. We cannot use [split] to
    # split the block into whitespace seperated strings as it
    # throws away the whitespace which maybe important so we
    # have to do it all by hand.

    set words {}
    set word ""

    while {[string length $block]} {
        # Look for the next group of whitespace characters.
        if {[regexp -indices "\[ \t\n\]+" $block all]} {
	    # Remove the text leading up to and including the white space
	    # from the block.
	    set text [string range $block 0 [lindex $all 1]]
	    set block [string range $block [expr {[lindex $all 1] + 1}] end]
	} else {
	    # Take everything up to the end of the block.
	    set text $block
	    set block {}
	}

	# Add the text to the end of the word we are building up.
        append word $text

        if { [catch {llength $word} length] == 0 && $length == 1} {
	    # The word is a valid list so add it to the list.
	    lappend words [string trim $word]
	    set word {}
        }
    }

    # If the last word has not been added to the list then there
    # is a problem.
    if { [string length $word] } {
	error "incomplete word \"$word\""
    }

    return $words
 }


#set TYPE {one_sample process_sample processing_nothing first_beat_skipped_processing heart_beat_detected eeg_peak}
set TYPE {processing_nothing.TC}

foreach curr_type $TYPE {
	set power_number_filename "hier_$curr_type.rpt"
	set result_filename "$curr_type.res"

	set power_number_file [open $curr_type/route_TC/hier.rpt r]
	set result_file [open $result_filename w]

	set linecount 0
	set is_count -1
	set print_line 0
	set w1 9
	set w2 34
	set w3 18
	set w4 14
	set w5 10
	
	set iso_power_switch 0
	set iso_power_int 0
	set iso_power_static 0
	set iso_power_total 0
	set set_iso_power 0
	set add_iso_power 0
	set print_iso 0
	set iso_rel_total_power 0
	
	puts $result_file "Mode: $curr_type"

	foreach line [split [read $power_number_file] \n] {

		set words [SplitIntoWords $line] 
		
		# get total leakage power as reference
		if {[string match duckweed_system_ic [lindex $words 0]] == 1} { 
			set total_leakage [lindex $words 3]
		}

		if {$linecount == 10} {
			set power1 [format "%${w2}s %${w1}s %${w1}s %${w1}s %${w1}s %${w1}s %${w1}s" "[lindex $words 0]" "[lindex $words 1]" "[lindex $words 2]" "[lindex $words 3]" "Total" "Leakage" "Leakage"]
		}

		if {$linecount == 11} {
			set power2 [format "%${w2}s %${w1}s %${w1}s %${w1}s %${w1}s %${w1}s %${w1}s" "[lindex $words 1]" "[lindex $words 2]" "[lindex $words 3]" "[lindex $words 4]" "[lindex $words 5]" "Total %" "DP %"] 
		}

		if {[string match i_duckweed [lindex $words 0]] == 1} {	
			puts $result_file "\n$power1"
			puts $result_file $power2
			puts $result_file "----------------------------------------------------------------------------------------------"		
			puts -nonewline $result_file "[lindex $words 0]\t\t"
			
			set rel_leakage [expr [lindex $words 4]/$total_leakage*100]
			
			puts $result_file [format "%${w3}s %${w1}s %${w1}s %${w1}s %${w1}s %${w1}.1f" "[lindex $words 2]" "[lindex $words 3]" "[lindex $words 4]" "[lindex $words 5]" "[lindex $words 6]" "$rel_leakage"] 
		}	

		if {[string match i_core [lindex $words 0]] == 1} {
			puts -nonewline $result_file "  [lindex $words 0]\t\t"
			set rel_leakage [expr [lindex $words 4]/$total_leakage*100]
			puts $result_file [format "%${w3}s %${w1}s %${w1}s %${w1}s %${w1}s %${w1}.1f" "[lindex $words 2]" "[lindex $words 3]" "[lindex $words 4]" "[lindex $words 5]" "[lindex $words 6]" "$rel_leakage"] 	
		}

		if {[string match i_dp [lindex $words 0]] == 1} {	
			puts -nonewline $result_file "    [lindex $words 0]\t\t"
			set rel_leakage [expr [lindex $words 4]/$total_leakage*100]
			set dp_leakage [lindex $words 4]
			puts $result_file [format "%${w3}s %${w1}s %${w1}s %${w1}s %${w1}s %${w1}.1f" "[lindex $words 2]" "[lindex $words 3]" "[lindex $words 4]" "[lindex $words 5]" "[lindex $words 6]" "$rel_leakage"] 
		}	

		if { $is_count != -1 && [string match i_dec [lindex $words 0]] == 1} {	
			puts -nonewline $result_file "        [lindex $words 0]\t\t"
			set rel_leakage [expr [lindex $words 4]/$total_leakage*100]
			set rel_leakage_dp [expr [lindex $words 4]/$dp_leakage*100]
			puts $result_file [format "%${w4}s %${w1}s %${w1}s %${w1}s %${w1}s %${w1}.1f %${w1}.1f" "[lindex $words 2]" "[lindex $words 3]" "[lindex $words 4]" "[lindex $words 5]" "[lindex $words 6]" "$rel_leakage" "$rel_leakage_dp"] 
		}

		if {$print_line == 1} {
			set rel_leakage [expr [lindex $words 2]/$total_leakage*100]
			set rel_leakage_dp [expr [lindex $words 2]/$dp_leakage*100]
			puts $result_file [format "%${w4}s %${w1}s %${w1}s %${w1}s %${w1}s %${w1}.1f %${w1}.1f" "[lindex $words 0]" "[lindex $words 1]" "[lindex $words 2]" "[lindex $words 3]" "[lindex $words 4]"  "$rel_leakage" "$rel_leakage_dp"] 
			set print_line 0
		}	
		
		if {$print_line == 2} {
			set rel_leakage [expr [lindex $words 2]/$total_leakage*100]
			set rel_leakage_dp [expr [lindex $words 2]/$dp_leakage*100]
			puts $result_file [format "%${w5}s %${w1}s %${w1}s %${w1}s %${w1}s %${w1}.1f %${w1}.1f" "[lindex $words 0]" "[lindex $words 1]" "[lindex $words 2]" "[lindex $words 3]" "[lindex $words 4]"  "$rel_leakage" "$rel_leakage_dp"] 
			set print_line 0
		}	
		
		if {$set_iso_power == 1} {
			set iso_power_switch [lindex $words 0]
			set iso_power_int [lindex $words 1]
			set iso_power_static [lindex $words 2]			
			set iso_power_total [lindex $words 3]
			set iso_rel_total_power [lindex $words 4]
			set set_iso_power 0
		}				
		
		if {$add_iso_power == 1} {
			set iso_power_switch [expr $iso_power_switch + [lindex $words 0]]
			set iso_power_int [expr $iso_power_int + [lindex $words 1]]
			set iso_power_static [expr $iso_power_static + [lindex $words 2]]
			set iso_power_total [expr $iso_power_total + [lindex $words 3]]
			set iso_rel_total_power [expr $iso_rel_total_power + [lindex $words 4]]
			set add_iso_power 0									
		}
		
		if {$print_iso == 1} {
			set rel_leakage [expr $iso_power_static/$total_leakage*100]
			set rel_leakage_dp [expr $iso_power_static/$dp_leakage*100]
			puts -nonewline $result_file "Total:\t\t\t\t\t"
			puts $result_file [format "%${w5}.2e %${w1}.2e %${w1}.2e %${w1}.2e %${w1}.1f %${w1}.1f %${w1}.1f" "$iso_power_switch" "$iso_power_int" "$iso_power_static" "$iso_power_total"  "$iso_rel_total_power" "$rel_leakage" "$rel_leakage_dp"] 
			set print_iso 0
		}		

		if {[string match i_exu_I* [lindex $words 0]] == 1} {	
			puts -nonewline $result_file "        [lindex $words 0]\t"	
			set print_line 1				
		}

		if {[string match i_rf_I* [lindex $words 0]] == 1} {
			puts $result_file "----------------------------------------------------------------------------------------------"	
			puts -nonewline $result_file "      [lindex $words 0]\t\t"	
			set print_line 1	
		}	

		if { [string match i_slot [lindex $words 0]] == 1} {	
			incr is_count
			puts $result_file "----------------------------------------------------------------------------------------------"
			puts -nonewline $result_file "      [lindex $words 0] ($is_count)\t"	
			set print_line 1	
		}
		
		if { [string match iso_is_eeg* [lindex $words 0]] == 1} {
			puts -nonewline $result_file "[lindex $words 0]\t"	
			set print_line 2	
			set add_iso_power 1
		}
		
		if { [string match RC_ISO_HIER_INST_2 [lindex $words 0]] == 1} {
			puts $result_file "----------------------------------------------------------------------------------------------"
			puts $result_file "----------------------------------------------------------------------------------------------"
			puts $result_file "ISOLATION CELLS"
			puts -nonewline $result_file "[lindex $words 0]\t"	
			set print_line 1
			set set_iso_power 1
		}		
		
		if { [string match RC_ISO_HIER_INST_12 [lindex $words 0]] == 1} {
			#puts $result_file "------------------------------------------------------------------------------------------"
			puts -nonewline $result_file "[lindex $words 0]\t"	
			set print_line 1	
			set add_iso_power 1	
			set print_iso 1		
		}		

		incr linecount

	}
	puts $result_file "----------------------------------------------------------------------------------------------"

	close $power_number_file
	close $result_file

}
