#!/usr/bin/tcl

proc SplitIntoWords {block} {

    # We need to split the block up into words but cannot use
    # list operations as they throw away some significant
    # quoting, and [split] ignores braces as it should.
    # Therefore what we do is gradually build up a string out of
    # whitespace seperated strings. We cannot use [split] to
    # split the block into whitespace seperated strings as it
    # throws away the whitespace which maybe important so we
    # have to do it all by hand.

    set words {}
    set word ""

    while {[string length $block]} {
        # Look for the next group of whitespace characters.
        if {[regexp -indices "\[ \t\n\]+" $block all]} {
	    # Remove the text leading up to and including the white space
	    # from the block.
	    set text [string range $block 0 [lindex $all 1]]
	    set block [string range $block [expr {[lindex $all 1] + 1}] end]
	} else {
	    # Take everything up to the end of the block.
	    set text $block
	    set block {}
	}

	# Add the text to the end of the word we are building up.
        append word $text

        if { [catch {llength $word} length] == 0 && $length == 1} {
	    # The word is a valid list so add it to the list.
	    lappend words [string trim $word]
	    set word {}
        }
    }

    # If the last word has not been added to the list then there
    # is a problem.
    if { [string length $word] } {
	error "incomplete word \"$word\""
    }

    return $words
 }


global test_app



global test_state 
global test_unit

global fu_is1_mul_pattern 0x80
global fu_is1_mul_on 1
global fu_vec_pattern 0x40
global fu_vec2_pattern 0x20
global VIS_pattern 0x2000
global IS2_pattern 0x10

set curr_unit_used 0
set curr_unit_on 1

set curr_unit $test_unit

upvar 0 ${curr_unit}_pattern curr_unit_pattern


foreach curr_state ${test_state} {
	foreach curr_test_unit ${test_unit} {

		set registerlog_filename "${test_app}_${curr_state}_register.log"
		set unit_result_filename "${test_app}_${curr_state}_${curr_test_unit}.dat"
		set results_filename "${test_app}_${curr_state}_${curr_test_unit}_over_time.txt"
		set units_filename "${test_app}_units_${curr_test_unit}.txt"

		source extract_units.tcl

		set registerlog [open $registerlog_filename r]
		set units_file [open $units_filename r]
		set results_file [open $results_filename w]
		set units_results_file [open $unit_result_filename w]

		set units [split [read $units_file] \n]

		set linecount 0

		foreach line [split [read $registerlog] \n] {

			set words [SplitIntoWords $line]
			set curr_unit_used 0
			
			# check if the current power domain is used
			if { [string is integer [lindex $words 0]] == 1 && [string length [lindex $words 0]] > 0} {
				set curr_PC [lindex [split [lindex $words 1] ()] 1]
				set curr_cycle [lindex $words 0]
				puts $results_file "PC: ${curr_PC}, cycle: [lindex $words 0] => [lindex $units ${curr_PC}]"
				set curr_line [SplitIntoWords [lindex $units ${curr_PC}]]
				if {[string length [lindex $units ${curr_PC}]] > 0} {
					set curr_unit_used 1
					puts -nonewline $units_results_file "1 "
				} else {
					set curr_unit_used 0
					puts -nonewline $units_results_file "0 "
				}		
			}
			
			# check if the current power domain is switched off
			if { [string equal [lindex $words 0] "PG"] == 1 } {
				set curr_PG [lindex $words 2]
				set curr_unit_check [expr $curr_PG&$curr_unit_pattern]
				if { $curr_unit_check == $curr_unit_pattern && $curr_unit_on == 1} {
					puts "$curr_unit is beeing switched off in cycle ${curr_cycle}"
					set curr_unit_on 0
				}
				if { $curr_unit_on == 0 && $curr_unit_check == 0} {
					puts "$curr_unit is beeing switched on in cycle ${curr_cycle}"
					set curr_unit_on 1
				}									
			}
			
			# check if current unit is scheduled although switched off
			if {$curr_unit_used == 1 && $curr_unit_on == 0} {
				puts "$curr_unit scheduled while switched off in cycle $curr_cycle, ERROR !!"
			}		
			
			incr linecount
		}

		close $registerlog
		close $units_file
		close $results_file
		close $units_results_file		
	}	
}

